"""Example."""

import logging

from pygleif import PyGleif

logging.basicConfig(level=logging.INFO)


LOGGER = logging.getLogger(__name__)

for lei_code in [
    "549300MLUDYVRQOOXS22",
    "M312WZV08Y7LYUC71685",
    "IGJSJL3JD5P30I6NJZ34",
    "3C7474T6CDKPR9K6YT90",
]:
    gleif: PyGleif = PyGleif(lei_code)
    message = (
        f"LEI: {lei_code} | Registration date: "
        f"{gleif.response.data.attributes.registration.initial_registration_date} | "
        f"Name: {gleif.response.data.attributes.entity.legal_name.name}"
    )

    LOGGER.info(message)



"""GLEIF API."""

from .api import GLEIFResponse
from .base import PyGleifBase


class PyGleif(PyGleifBase):
    """Query GLEIF API and return response."""

    def __init__(self, lei_code: str) -> None:
        """Init class."""
        self.search_string = lei_code

    @property
    def response(self) -> GLEIFResponse:
        """Return response."""
        return GLEIFResponse(**self.json_response)



"""Tests."""

from typing import Any
import pytest

from pygleif import PyGleif
import json
from unittest.mock import patch, PropertyMock
from collections.abc import Generator


def load_mock_data(file_name: str) -> dict[Any, Any]:
    """Load mock data from file."""
    with open(f"tests/fixtures/{file_name}.json") as file:
        return json.load(file)


@pytest.fixture
def fixture_response(request) -> Generator[Any, Any, Any]:
    """Mock data for the security service endpoint."""
    with patch(
        "pygleif.base.PyGleifBase.json_response", new_callable=PropertyMock
    ) as mock_response:
        mock_response.return_value = load_mock_data(request.param)
        yield request.param


@pytest.mark.parametrize(
    "fixture_response",
    ["9845001B2AD43E664E58_issued", "549300LBI3LRIZ2V8V66_lapsed"],
    indirect=True,
)
def test_pygleif(fixture_response) -> None:
    """Test Pygleif-class."""
    lei = fixture_response.split("_")[0]
    data = PyGleif(lei)
    assert data.response.data.attributes.lei == lei



"""Data."""

from datetime import datetime
from typing import Any

from pydantic import Field

from .shared import BaseSchema

DESCRIPTION_VALUES: dict[str, str] = {
    "lei": (
        "The LEI code is a 20-character, alpha-numeric code that defines the ISO "
        "17442 compatible identifier for the legal entity."
    ),
}


class ValidatedAt(BaseSchema):
    """Represent validated at information."""

    id: str
    other: str | None


class Registration(BaseSchema):
    """Represent registration information."""

    corroboration_level: str
    initial_registration_date: datetime
    last_update_date: datetime
    managing_lou: str
    next_renewal_date: datetime
    other_validation_authorities: list[Any]
    status: str
    validated_as: str | None = None
    validated_at: ValidatedAt


class GeneralEntity(BaseSchema):
    """Represent a general entity ."""

    lei: str | None = Field(..., description=DESCRIPTION_VALUES["lei"])
    name: str | None


class Address(BaseSchema):
    """Represent address ."""

    language: str | None
    address_lines: list[str]
    address_number: str | None
    address_number_within_building: str | None
    mail_routing: str | None
    city: str
    region: str | None
    country: str
    postal_code: str | None


class Expiration(BaseSchema):
    """Represent expiration data."""

    date: str | None
    reason: str | None


class LegalForm(BaseSchema):
    """Represent the legal form."""

    id: str | None
    other: str | None


class Name(BaseSchema):
    """Represent the name."""

    name: str | None
    language: str | None = None
    type: str | None = None


class RegisteredAt(BaseSchema):
    """Represent registered at."""

    id: str
    other: str | None


class Entity(BaseSchema):
    """Represent entity information."""

    associated_entity: GeneralEntity
    category: str
    creation_date: str | None
    event_groups: list[Any]
    expiration: Expiration
    headquarters_address: Address
    jurisdiction: str
    legal_address: Address
    legal_form: LegalForm
    legal_name: Name
    other_addresses: list[Any]
    other_names: list[Name]
    registered_as: str | None = None
    registered_at: RegisteredAt
    status: str
    successor_entities: list[Any]
    sub_category: str | None = None
    successor_entity: GeneralEntity
    transliteraded_other_names: list[Any] | None = None


class Attributes(BaseSchema):
    """Represent attribute information."""

    bic: list[str] | None
    lei: str = Field(..., description=DESCRIPTION_VALUES["lei"])
    entity: Entity
    registration: Registration


class LinkData(BaseSchema):
    """Represent a link ."""

    self: str | None
    related: str | None = None
    reporting_exception: str | None = None


class Links(BaseSchema):
    """Represent a link ."""

    links: LinkData


class Relationships(BaseSchema):
    """Represent a relationships ."""

    managing_lou: Links | None = None
    lei_issuer: Links | None = None
    field_modifications: Links | None = None
    direct_parent: Links | None = None
    ultimate_parent: Links | None = None


class Data(BaseSchema):
    """Represent data information."""

    id: str
    type: str
    attributes: Attributes
    links: LinkData
    relationships: Relationships



"""Search."""

from __future__ import annotations

from urllib import parse

from pygleif.api import SearchResponse

from .base import PyGleifBase


class Search(PyGleifBase):
    """Class to use the search form of the GLEIF web site."""

    def __init__(self, orgnr: str) -> None:
        """Init class."""
        encoded_org_nr = parse.quote(orgnr, safe="")
        self.search_string = f"?filter[fulltext]={encoded_org_nr}"

    @property
    def response(self) -> SearchResponse | None:
        """Return response."""
        return SearchResponse(**self.json_response)



"""Meta data."""

from datetime import datetime

from .shared import BaseSchema


class GoldenCopy(BaseSchema):
    """Represent golden copy information."""

    publish_date: datetime


class Pagination(BaseSchema):
    """Represent response pagination."""

    current_page: int
    per_page: int
    _from: int
    to: int | None = None
    total: int
    last_page: int


class Meta(BaseSchema):
    """Represent meta information."""

    golden_copy: GoldenCopy
    pagination: Pagination | None = None



"""A base class."""

from __future__ import annotations

from abc import ABC, abstractmethod
from enum import IntEnum
import json
from typing import TYPE_CHECKING, Any, cast
from urllib import error, request

from pygleif.error import PyGLEIFApiError

if TYPE_CHECKING:
    from .api import GLEIFResponse, SearchResponse


class HttpErrorCodes(IntEnum):
    """Represent HTTP error codes."""

    NOT_FOUND = 404


class PyGleifBase(ABC):
    """Base class for a GLEIF API request."""

    TIMEOUT_SECOND = 10  # 10 seconds

    search_string: str

    @property
    def json_response(self) -> dict[Any, Any]:
        """Return JSON response."""
        try:
            with request.urlopen(
                f"https://api.gleif.org/api/v1/lei-records/{self.search_string}",
                timeout=self.TIMEOUT_SECOND,
            ) as response:
                return cast("dict[Any, Any]", json.loads(response.read()))
        except error.HTTPError as e:
            if e.code == HttpErrorCodes.NOT_FOUND:
                msg = "Resource not found"
                raise PyGLEIFApiError(msg) from e

            msg = f"HTTP Error encountered: {e.code}"
            raise PyGLEIFApiError(msg) from e
        except error.URLError as e:
            msg = f"URL Error encountered: {e.reason}"
            raise PyGLEIFApiError(msg) from e
        except Exception as e:
            msg = f"An unexpected error occurred: {e!s}"
            raise PyGLEIFApiError(msg) from e

    @property
    @abstractmethod
    def response(self) -> GLEIFResponse | SearchResponse | None:
        """Return response."""



"""Tests for search function."""

import pytest

from pygleif import Search


@pytest.fixture(scope="module", name="gleif_search_fixture")
def search_data() -> Search:
    """Fixture."""
    return Search("917685991")


def test_search_lei(gleif_search_fixture: Search):
    """Test LEI attribute."""
    assert gleif_search_fixture.response.data[0].attributes.lei, "549300MLUDYVRQOOXS22"


def test_search_id(gleif_search_fixture: Search):
    """Test ID attribute."""
    assert gleif_search_fixture.response.data[0].id, "549300MLUDYVRQOOXS22"


def test_search_no_result():
    """Test failing search."""
    search = Search("1bcdefghijklmnopqrstuvwxyz")
    assert search.response.data == []



"""Pydantic Representation of models."""

from .data import Data
from .meta import Meta
from .shared import BaseSchema


class GLEIFResponse(BaseSchema):
    """Represent a base response."""

    meta: Meta
    data: Data


class SearchResponse(BaseSchema):
    """Represent search result response."""

    meta: Meta
    data: list[Data]
